unit hiZipInfo;

interface

uses kol,windows,ShellAPI,Share,Debug;

type ZipFile = Record
  Version                      : WORD;
  Flag                         : WORD;
  CompressionMethod            : WORD;
  Time                         : WORD;
  Date                         : WORD;
  CRC32                        : Longint;
  CompressedSize               : Longint;
  UncompressedSize             : Longint;
  FileNameLength               : WORD;
  ExtraFieldLength             : WORD;
  FileName                     : String;
end;

type PZipFile = ^ZipFile;

const
  LocalFileHeaderSig    = $04034b50;
  CentralFileHeaderSig  = $02014b50;
  EndCentralDirSig      = $06054b50;
  //soFromCurrent         = 1;
  ThousandSeparator     = ' ';

type TReadFile = procedure (ZipFile : ZipFile) of object;

type
  THIZipInfo = class(TDebug)
  public
    constructor Create;
    destructor Destroy;override;

    function GetDate(Date:Word):TDateTime;
    procedure Read(Value:String);

   public
    _event_onNext:THI_Event;
    _event_onEnd:THI_Event;

    procedure _work_doReadFileZip(var _Data:TData; Index:word);
    procedure _var_FileName(var _Data:TData; Index:word);
    procedure _var_DateTime(var _Data:TData; Index:word);
    procedure _var_Packed(var _Data:TData; Index:word);
    procedure _var_UnPacked(var _Data:TData; Index:word);
    procedure _var_CompressionMethod(var _Data:TData; Index:word);
    procedure _var_CRC32(var _Data:TData; Index:word);
    procedure _var_FileTypeInfo(var _Data:TData; Index:word);
  end;

implementation

var ZFile: ZipFile;

function GetFileTypeInfo(FileName: string): string;
var
  ShFileInfo: TShFileInfo;
  TypeInfo: string[79];
begin
  ShGetFileInfo(PChar(FileName), 0, ShFileInfo , SizeOf(TShFileInfo),
  SHGFI_SYSICONINDEX or  SHGFI_TYPENAME or SHGFI_USEFILEATTRIBUTES);
  TypeInfo := ShFileInfo.szTypeName;
  Result := TypeInfo;
end;

function Frmt(ts: string): string;
var
  a: Integer;
begin
  a := 3;
  while (a < length(ts)) do begin
    Insert(ThousandSeparator, ts, Length(ts) - a + 1); a := a + 4;
  end;
  Result := ts;
end;

constructor THIZipInfo.Create;
begin
inherited Create;
end;

Destructor THIZipInfo.Destroy;
begin
  inherited Destroy;
end;

function THIZipInfo.GetDate(Date:Word):TDateTime;
var FileTime:TFileTime;
    SysTime:TSystemTime;
    D,M,Y:Word;
    Time:Word;
begin
  Time := 0;
  DosDateTimeToFileTime(Date,Time,FILETIME);
  FileTimeToSystemTime(FileTime,SysTime);
  Y:=SysTime.wYear;
  M:=SysTime.wMonth;
  D:=SysTime.wDay;
  EncodeDate(Y,M,D,Result);
end;

procedure THIZipInfo.Read(Value:String);
var Sig: Longint;
    ZipStream: PStream;
    Res: Longint;
    Name: array [0..255] of char;
    tmpCount: Integer;
begin
tmpCount:=0;
ZipStream:=NewReadFileStream(Value);
while TRUE do
begin
  inc(tmpCount);
  Res := ZipStream.Read(Sig,SizeOf(Sig));
  if (Res = HFILE_ERROR) or (Res <> SizeOf(Sig)) then
    Break
  else

  if Sig = LocalFileHeaderSig then
  begin

    ZipStream.Read(ZFile.Version,SizeOf(WORD));
    ZipStream.Read(ZFile.Flag,SizeOf(WORD));
    ZipStream.Read(ZFile.CompressionMethod,SizeOf(WORD));
    ZipStream.Read(ZFile.Time,SizeOf(WORD));
    ZipStream.Read(ZFile.Date,SizeOf(WORD));
    ZipStream.Read(ZFile.CRC32,SizeOf(Longint));
    ZipStream.Read(ZFile.CompressedSize,SizeOf(Longint));
    ZipStream.Read(ZFile.UncompressedSize,SizeOf(Longint));
    ZipStream.Read(ZFile.FileNameLength,SizeOf(WORD));
    ZipStream.Read(ZFile.ExtraFieldLength,SizeOf(WORD));
    ZipStream.Read(Name,ZFile.FileNameLength);
    ZFile.FileName:=Copy(Name,0,ZFile.FileNameLength);
    ZipStream.Seek(ZFile.ExtraFieldLength,{soFromCurrent}spCurrent);
    ZipStream.Seek(ZFile.CompressedSize,{soFromCurrent}spCurrent);
    
    _hi_OnEvent(_event_onNext, tmpCount);

  end
  else
  if Sig = CentralFileHeaderSig then
  begin
    ZipStream.Seek(42,spCurrent)
  end
  else
  if Sig = EndCentralDirSig then
    Break;
end;
ZipStream.Free;
_hi_OnEvent(_event_onEnd, 1);
end;

procedure THIZipInfo._work_doReadFileZip;
var Value: string;
begin
  Value := ToString(_Data);
  if FileExists(Value) then
  Read(Value);
end;

procedure THIZipInfo._var_FileName;
var N: PChar;
begin
  Replace(ZFile.FileName,'/','\');
  N := PChar(ZFile.FileName);
  OemToChar(N, N);
  dtString(_Data, N);
end;

procedure THIZipInfo._var_DateTime;
var dt: TDateTime;
begin
  dt := GetDate(ZFile.Date);
  dtString(_Data, Date2StrFmt('dd-MM-yyyy',dt));
end;

procedure THIZipInfo._var_Packed;
begin
  dtString(_Data, Frmt(Int2Str(ZFile.CompressedSize)));
end;

procedure THIZipInfo._var_UnPacked;
begin
  dtString(_Data, Frmt(Int2Str(ZFile.UnCompressedSize)));
end;

procedure THIZipInfo._var_CompressionMethod;
var tmp: integer;
begin
 tmp := ZFile.CompressionMethod;
 case tmp of
   0: dtString(_Data, 'stored'); //(no compression)
   1: dtString(_Data, 'Shrunk');
   2: dtString(_Data, 'factor1');
   3: dtString(_Data, 'factor2');
   4: dtString(_Data, 'factor3');
   5: dtString(_Data, 'factor4');
   6: dtString(_Data, 'Imploded');
   //7 - Reserved for Tokenizing compression algorithm
   8: dtString(_Data, 'Deflated');
 end;
end;

procedure THIZipInfo._var_CRC32;
begin
  dtString(_Data, Int2Hex(ZFile.CRC32, 8));
end;

procedure THIZipInfo._var_FileTypeInfo;
begin
  dtString(_Data, GetFileTypeInfo(ExtractFileName(ZFile.FileName)));
end;

end.
